#!/usr/bin/python3
# coverage-builder tests
# (C) 2014 Canonical Ltd.
# Author: Allan LeSage <allan.lesage@canonical.com>
# License: GPL 3 or later

import re
import os
import shutil
import subprocess
import tempfile
import unittest


CURRENTLY_BUILDING_TEXT = """Package: {package}
Component: {component}
Suite: {suite}
Purpose: {purpose}
"""


def snip_coverage_xml(log_text):
    """Extract coverage.xml from a specially-marked log.

    NOTE that log_text must be an ascii string.
    """
    regex = re.compile(
        """===== BEGIN coverage.xml =====
(.+)
===== END coverage.xml =====""",
        re.DOTALL)
    match = regex.search(log_text)
    try:
        return match.group(1)
    except AttributeError:
        return None


def snip_coverage_xml_from_log(log_filepath):
    with open(log_filepath) as log_file:
        log_text = log_file.read()
        return snip_coverage_xml(log_text)


class CoverageBuilderTestCase(unittest.TestCase):

    def _symlink(self, local_dh_script):
        os.symlink(
            os.path.join(
                self.srcdir,
                'src',
                local_dh_script),
            os.path.join(
                self.srcdir,
                'coverage-builder',
                local_dh_script))

    def _unlink(self, local_dh_script):
        os.unlink(
            os.path.join(
                self.srcdir,
                'coverage-builder',
                local_dh_script))

    def _copy_dummy_project_into_place(self, project_name):
        shutil.copytree(
            os.path.join(
                self.srcdir,
                'test',
                project_name),
            self.pkgdir)

    def _find_gcno(self):
        '''Return True if a .gcno is found in pkgdir.'''
        for dirpath, dirnames, files in os.walk(self.pkgdir):
            for filename in files:
                root, ext = os.path.splitext(filename)
                if ext == '.gcno':
                    return True
        return False

    def _fake_currently_building(self,
                                 package='fibonacci',
                                 component='main',
                                 suite='utopic',
                                 purpose='PPA'):
        """Deposit a fake /CurrentlyBuilding file.

        This enables us to pretend to build in a PPA (or not).  NOTE
        that we stash the fake path of this file, it's used in dh_
        scripts.
        """
        currently_building_text = CURRENTLY_BUILDING_TEXT.format(
            package=package,
            component=component,
            suite=suite,
            purpose=purpose)
        # stash this path b/c we'll have to update env with it
        self.currently_building_filepath = os.path.join(
            self.workdir, 'CurrentlyBuilding')
        with open(self.currently_building_filepath, 'w') as f:
            f.write(currently_building_text)

    def _update_env_with_currently_building_path(self, env):
        """Export the path of our /CurrentlyBuilding file, return env.

        NOTE that this path will be used by dh_ scripts.
        """
        if env is None:
            env = os.environ.copy()
        if env.get('CURRENTLY_BUILDING_PATH') is None:
            env['CURRENTLY_BUILDING_PATH'] = self.currently_building_filepath
        return env

    def setUp(self):
        super(CoverageBuilderTestCase, self).setUp()

        self.srcdir = os.path.dirname(
            os.path.dirname(os.path.abspath(__file__)))
        self.workdir = tempfile.mkdtemp()
        self.pkgdir = os.path.join(self.workdir, 'dummy')

        os.mkdir(os.path.join(self.srcdir, 'coverage-builder'))
        # add this new dir to our path so that our special dh_ is called first
        os.environ['PATH'] = os.path.join(
            self.srcdir, 'coverage-builder') + os.pathsep + os.environ['PATH']
        # for the term of this test let's use our local dh_ scripts
        for script in ['dh_auto_configure',
                       'dh_auto_build',
                       'dh_auto_test',
                       'coverage_activated']:
            self._symlink(script)

        self._fake_currently_building()
        os.environ['COVERAGE_ACTIVATED_PATH'] = os.path.join(
            self.srcdir, 'coverage-builder')

    def tearDown(self):
        shutil.rmtree(self.workdir)
        for script in ['dh_auto_configure',
                       'dh_auto_build',
                       'dh_auto_test',
                       'coverage_activated']:
            self._unlink(script)
        os.rmdir(os.path.join(self.srcdir, 'coverage-builder'))
        os.environ['PATH'] = os.environ['PATH'].replace(
            os.path.join(self.srcdir, 'coverage-builder') + os.pathsep,
            '')
        super(CoverageBuilderTestCase, self).tearDown()

    def _call_dh_auto_configure(self, env=None):
        which_dh_auto_configure = shutil.which('dh_auto_configure')
        self.assertEqual(
            os.path.join(self.srcdir, 'coverage-builder', 'dh_auto_configure'),
            which_dh_auto_configure)
        env = self._update_env_with_currently_building_path(env)
        configure = subprocess.Popen(
            ['dh_auto_configure'],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            cwd=self.pkgdir,
            env=env)
        return configure.communicate()

    def _call_dh_auto_build(self, env=None):
        which_dh_auto_build = shutil.which('dh_auto_build')
        self.assertEqual(
            os.path.join(self.srcdir, 'coverage-builder', 'dh_auto_build'),
            which_dh_auto_build)
        env = self._update_env_with_currently_building_path(env)
        build = subprocess.Popen(
            ['dh_auto_build'],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            cwd=self.pkgdir,
            env=env)
        return build.communicate()

    def _call_dh_auto_test(self):
        which_dh_auto_test = shutil.which('dh_auto_test')
        self.assertEqual(
            os.path.join(self.srcdir, 'coverage-builder', 'dh_auto_test'),
            which_dh_auto_test)
        test = subprocess.Popen(
            ['dh_auto_test'],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            cwd=self.pkgdir)
        return test.communicate()

    def _call_coverage_activated(self, env=None):
        which_coverage_activated = shutil.which('coverage_activated')
        self.assertEqual(
            os.path.join(self.srcdir, 'coverage-builder', 'coverage_activated'),
            which_coverage_activated)
        env = self._update_env_with_currently_building_path(env)
        coverage_activated = subprocess.Popen(
            ['coverage_activated'],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            cwd=self.pkgdir,
            env=env)
        out, err = coverage_activated.communicate()
        return coverage_activated.returncode
